import 'dart:async';

import 'package:flame/components.dart';
import 'package:flame/extensions.dart';
import 'package:flame/game.dart';
import 'package:flame/palette.dart';
import 'package:flame_fast_touch/flame_fast_touch.dart';
import 'package:flame_spatial_grid/flame_spatial_grid.dart';

const _debugEnabled = true;

class MinimalGame extends FlameGame with HasSpatialGridFramework, FastTouch {
  MinimalGame();

  late final CameraComponent cameraComponent;
  late final JoystickComponent joystick;

  @override
  FutureOr<void> onLoad() async {
    _createJoystick();

    final player = Player(position: Vector2.zero());

    camera.viewport.addAll([joystick, FpsTextComponent()]);
    componentsAtPointRoot = joystick;

    await initializeSpatialGrid(
      cellSize: 50,
      // activeRadius: const Size.square(2),
      // preloadRadius: const Size.square(2),
      // unloadRadius: const Size.square(2),
      rootComponent: world,
      trackedComponent: SpatialGridDebugCameraWrapper(camera),
      trackWindowSize: false,
      processCellsLimitToPauseEngine: 1000,
      debug: _debugEnabled,
    );

    world.add(player);
    camera.viewfinder.zoom = 1;
    camera.priority = 999;
    camera.follow(player, maxSpeed: 200, snap: true);

    return super.onLoad();
  }

  void _createJoystick() {
    joystick = JoystickComponent(
      knob: CircleComponent(
        radius: 25,
        paint: BasicPalette.lightBlue.paint(),
      ),
      background: CircleComponent(
        radius: 100,
        paint: BasicPalette.blue.paint(),
      ),
      position: Vector2.all(100),
    );
  }
}

class SpatialGridDebugCameraWrapper extends SpatialGridCameraWrapper {
  SpatialGridDebugCameraWrapper(super.cameraComponent);

  @override
  void onAfterZoom() {
    try {
      if (!game.isSpatialGridDebugEnabled) {
        game.onAfterZoom();
      }
      // ignore: avoid_catches_without_on_clauses, empty_catches
    } catch (e) {}
  }
}

class Player extends PositionComponent
    with HasGridSupport, HasPaint, HasGameRef<MinimalGame> {
  Player({super.position})
      : super(
          size: Vector2(25, 25),
          anchor: Anchor.center,
        ) {
    paint.color = BasicPalette.yellow.color;
    debugMode = _debugEnabled;
  }

  late final _rect = Rect.fromLTWH(0, 0, size.x, size.y);

  @override
  void render(Canvas canvas) {
    canvas.drawRect(_rect, paint);
  }

  @override
  void update(double dt) {
    super.update(dt);
    if (!game.joystick.delta.isZero()) {
      position.add(
        game.joystick.relativeDelta * 100 * dt,
      );
    }
  }
}

class MyWorld extends World {}
